/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

package io;

import java.io.IOException;
import java.util.Vector;

import javax.xml.bind.JAXBException;

import app.SEVinoView;

import lib.AuthorInfo;
import lib.BarElement;
import lib.ColorRectangle;
import lib.EffectElement;
import lib.EffectPanel;
import lib.EffectTrack;
import xml.SEMParser;

/**
 * Class for loading a SEM file
 * 
 * @author Markus Waltl
 * @version 1.0
 */
public class LoadSEM {
	private AuthorInfo authorInfo = null;
	private Vector<EffectTrack> effectTracks = null;
	
	/**
	 * Constructor
	 * @param effectTracks Vector with effect tracks to be filled
	 */
	public LoadSEM(Vector<EffectTrack> effectTracks) {
		this.effectTracks = effectTracks;
	}
	
	/**
	 * Method for loading a given SEM file 
	 * @param semFile String with path and name of SEM file
	 * @return true if loading successful else false
	 */
	@SuppressWarnings("unchecked")
	public boolean loadSEM(String semFile, SEVinoView gui) {
		boolean retVal = true;
		try {			
			// load file
			SEMParser parser = new SEMParser();
			Object[] semContent = (Object[])parser.parse(semFile);
			
			if (semContent != null) {
				if (semContent[0] != null)
					authorInfo = (AuthorInfo)semContent[0];
				
				if (semContent[1] != null) {
					Vector<EffectElement> effectElements = (Vector<EffectElement>)semContent[1];
					
					for (int i = 0; i < effectTracks.size(); i++) {
						EffectPanel effectPanel = effectTracks.elementAt(i).getEffectBarPanel();
						int counter = 0;
						for (int j = 0; j < effectElements.size(); j++) {
							EffectElement effectElement = effectElements.elementAt(j);
							if (effectPanel.getNameOfEffect() != null &&
									effectPanel.getNameOfEffect().equals(effectElement.getEffectName())) {
								
								if (effectElement.getEndTime() == effectElement.getStartTime() ||
										effectElement.getEndTime() <= 0.0) {
									// if we have no end-tag we will use the length of the video
									String durationLabel = gui.durationLabel.getText();
									String[] dur = durationLabel.split(":");
							        effectElement.setEndTime(((Double.valueOf(dur[0])*60)+Double.valueOf(dur[1]))*60+Double.valueOf(dur[2]));
								}
								
								ColorRectangle colorRectangle = new ColorRectangle((int)(effectElement.getStartTime()*70),
										effectPanel.POSY, (int)((effectElement.getEndTime()-effectElement.getStartTime())*70),
										effectPanel.HEIGHT, effectPanel.getColor(), "rect" + counter);
								
								effectElement.setConfigElement(effectPanel.getConfigElement());
								BarElement barElement = new BarElement(colorRectangle, effectElement);
								effectPanel.addBar(barElement);
								counter++;
							}
						}
					}
				}
				else
					retVal = false;
			}
			else
				retVal = false;
		} catch (IOException e) {
			System.out.println(e.getMessage());
			retVal = false;
		} catch (JAXBException e) {
			System.out.println(e.getMessage());
			retVal = false;
		} catch (Exception e) {
			System.out.println("An unexpected error occured!");
			retVal = false;
		}
		
		return retVal;
	}
	
	/**
	 * Method for retrieving the author information loaded
	 * @return AuthorInfo with found content
	 */
	public AuthorInfo getAuthorInfo() {
		return authorInfo;
	}
	
	/**
	 * Method for returning the filled effect tracks
	 * @return Vector<EffectTrack> with filled effect tracks
	 */
	public Vector<EffectTrack> getEffectTracks() {
		return effectTracks;
	}
}
