/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

package io;

import lib.AuthorInfo;
import lib.BarElement;
import lib.EffectElement;
import lib.HelperFunctions;
import lib.SEMDefinitions;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;
import javax.xml.bind.JAXBException;

import xml.SEMCreator;

/**
 * Class for generating a SEM file
 * 
 * @author Markus Waltl
 * @version 1.0
 */
public class SaveSEM {
	private AuthorInfo authorInfo = null;
	private Hashtable<String, Vector<BarElement>> effectTracks = null;
	private String outputFile = "";
	private final static long PTS = 90000;
	
	/**
	 * Constructor
	 * @param ai AuthorInfo to be used
	 * @param tracks Hashtable<String, Vector<BarElement>> to be filled
	 * @param file String with path and name of file to generate
	 */
	public SaveSEM(AuthorInfo ai, Hashtable<String, Vector<BarElement>> tracks, String file) {
		authorInfo = ai;
		effectTracks = tracks;
		outputFile = file;
	}
	
	/**
	 * Method for generating the SEM file
	 * @return true if generated successful else false
	 */
	public boolean generateSEM() {
		return generateSEM(null, 0, null, false);
	}
	
	/**
	 * Method for generating and uploading the SEM file
	 * @param server address to the server
	 * @param port int with the port number
	 * @param path path on the server
	 * @param upload true if files should be uploaded else false
	 * @return true if generated successful else false
	 */
	public boolean generateSEM(String server, int port, String path, boolean upload) {
		boolean retVal = true;
		
		if (authorInfo != null) {
			DateFormat dfmt = new SimpleDateFormat("yyyy'-'dd'-'MM'T'hh:mm:ss"); 
			String date = dfmt.format(new Date());
			if (authorInfo.getCreationTime().equals(""))
				authorInfo.setCreationTime(date);
			
			authorInfo.setLastUpdate(date);
		}
		
		try {
			SEMCreator sc = new SEMCreator();

			
			/*sc.setSEMAttributes("urn:mpeg:mpeg-v:2010:01-SEDL-NS", "xmlns:sedl='urn:mpeg:mpeg-v:2010:01-SEDL-NS' " +
				"xmlns:sev='urn:mpeg:mpeg-v:2010:01-SEV-NS' " +
				"xmlns:si='urn:mpeg:mpeg21:2003:01-DIA-XSI-NS' " +
				"xmlns:mpeg7='urn:mpeg:mpeg7:schema:2004' si:absTimeScheme='mp7t' " +
				"si:puMode='ancestorsDescendants' si:timeScale='" + PTS + "'");
			*/
			sc.setSEMAttributes("urn:mpeg:mpeg-v:2010:01-SEDL-NS", 
					"xmlns:si='urn:mpeg:mpeg21:2003:01-DIA-XSI-NS' " +
					"si:absTimeScheme='mp7t' " +
					"si:puMode='ancestorsDescendants' si:timeScale='" + PTS + "'");
			
			if (authorInfo != null)
				sc.setDescriptionMetadata("urn:mpeg:mpeg-v:2010:01-SEDL-NS", authorInfo.getStringRepresentation().toString());
			
			
			Hashtable<BigDecimal, Vector<EffectElement>> mergedElements = mergeEffectElements();
			Vector<BigDecimal> v = new Vector<BigDecimal>(mergedElements.keySet());
			Collections.sort(v);
			Iterator<BigDecimal> it = v.iterator();
			StringBuffer sb = new StringBuffer();
			while (it.hasNext()) {
				BigDecimal key = it.next();
				Vector<EffectElement> vEE = mergedElements.get(key);
				// check if we need GroupOfEffects
				boolean GOE = false;
				if (vEE.size() > 1) {
					sb.append("<GroupOfEffects xmlns='urn:mpeg:mpeg-v:2010:01-SEDL-NS' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:sev='urn:mpeg:mpeg-v:2010:01-SEV-NS' xmlns:si='urn:mpeg:mpeg21:2003:01-DIA-XSI-NS' " +
							"si:anchorElement='true' si:pts='" + key.longValue() + "'>");
					GOE = true;
				}
				
				for (int i = 0; i < vEE.size(); i++) {
					EffectElement ee = vEE.elementAt(i);
					boolean deactivate = false;
					sb.append("<Effect xmlns='urn:mpeg:mpeg-v:2010:01-SEDL-NS' xmlns:sev='urn:mpeg:mpeg-v:2010:01-SEV-NS' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' " +
							"xsi:type='sev:" + ee.getEffectName() + "' activate='");
					
					BigDecimal startTime = HelperFunctions.round(ee.getStartTime(), 1);
					startTime = startTime.multiply(new BigDecimal(PTS));
									
					if (startTime.compareTo(key) == 0)
						sb.append("true'");
					else {
						sb.append("false'");
						deactivate = true;
					}
					
					if (!deactivate) {
						Vector<String[]> attributes = ee.getAttributes();
						for (int j = 0; j < attributes.size(); j++) {
							String[] attribute = attributes.elementAt(j);
						
							// check if name and value are not empty
							if (!attribute[0].equals("") && !attribute[1].equals("")) {
								// autoExtraction has extra entry "no" for defining no auto extraction method
								if (attribute[0].equals(SEMDefinitions.AUTOEXTRACTION)) {
									if (!attribute[1].equalsIgnoreCase("no"))
										sb.append(" " + attribute[0] + "='" + attribute[1] + "'");
								}
								else if (attribute[0].equals(SEMDefinitions.DURATION) || attribute[0].equals(SEMDefinitions.ALT) || 
										attribute[0].equals(SEMDefinitions.INTENSITY) ||
										attribute[0].equals(SEMDefinitions.LOCATION) ||
										attribute[0].equals(SEMDefinitions.PRIORITY)) { //|| attribute[0].equals(SEMDefinitions.ACTIVATE)) // activate not needed because handled separately
									if (attribute[0].equals(SEMDefinitions.INTENSITY) || !attribute[1].equals("-1"))
										sb.append(" " + attribute[0] + "='" + attribute[1] + "'");
								}
								else if (attribute[0].equals(SEMDefinitions.FADE)) {
									BigDecimal fadeTime = new BigDecimal(attribute[1]);
									fadeTime = HelperFunctions.round(fadeTime, 1);
									fadeTime = fadeTime.multiply(new BigDecimal(PTS));
									if (fadeTime.longValue() > 0)
										sb.append(" " + attribute[0] + "='" + fadeTime.longValue() + "'");
								}
								else {
									if (!attribute[1].equals("-1"))
										sb.append(" sev:" + attribute[0] + "='" + attribute[1] + "'");
								}
							}
						}
					}
					
					if (!GOE) {
						sb.append(" xmlns:si='urn:mpeg:mpeg21:2003:01-DIA-XSI-NS' si:anchorElement='true' si:pts='" + key.longValue() + "'/>");
					}
					else 
						sb.append("/>");
				}
				
				if (GOE) {
					sb.append("</GroupOfEffects>");
					//System.out.println("GOE: " + sb.toString());
					sc.addGroupOfEffects("urn:mpeg:mpeg-v:2010:01-SEDL-NS", sb.toString());
				}
				else {
					//System.out.println("E: " + sb.toString());
					sc.addEffect("urn:mpeg:mpeg-v:2010:01-SEDL-NS", sb.toString());
				}
				
				sb = new StringBuffer();
			}
				
			String schemaLocation = "urn:mpeg:mpeg21:2003:01-DIA-XSI-NS http://standards.iso.org/ittf/PubliclyAvailableStandards/MPEG-21_schema_files/dia-2nd/XSI-2nd.xsd" +
				" urn:mpeg:mpeg-v:2010:01-SEV-NS http://standards.iso.org/ittf/PubliclyAvailableStandards/MPEG-V_schema_files/MPEG-V-SEV.xsd" +
				" urn:mpeg:mpeg7:schema:2004 http://standards.iso.org/ittf/PubliclyAvailableStandards/MPEG-7_schema_files/mpeg7-v2.xsd";
			
			String styleSheet = "";
			String OS = System.getProperty("os.name").toLowerCase();
			if (OS.indexOf("windows") > -1)
				styleSheet = "configs\\remNNAttributes.xsl";//, "configs\\schematron_validation.xsl";
			else
				styleSheet = "configs/remNNAttributes.xsl";//, "configs/schematron_validation.xsl";
			
			if (!upload) {
				sc.createSEM(outputFile, schemaLocation, styleSheet);
			}
			else { 
				if (upload && server != null && !server.equals("")) {
					sc.uploadSEM(server, port, path, outputFile, schemaLocation, styleSheet);
				}
				else {
					System.out.println("Cannot upload! Perhaps missing or incorrect server address!");
					retVal = false;
				}
			}
		} catch (JAXBException e) {
			System.out.println("Creation error: " + e.getMessage());
			retVal = false;
		} catch (Exception e) {
			System.out.println("Unexpected error: " + e.getMessage());
			retVal = false;
		}
		
		return retVal;
	}

	/**
	 * Method for merging the effect elements
	 * used for bringing all element in the same vector with the same start or end time
	 * which is used for building GroupOfEffects and for easier avoidance of useless deactivation
	 * @return Hashtable<BigDecimal, Vector<EffectElement>> with merged effect elements
	 */
	private Hashtable<BigDecimal, Vector<EffectElement>> mergeEffectElements() {
		Hashtable<BigDecimal, Vector<EffectElement>> mergedHashtable = new Hashtable<BigDecimal, Vector<EffectElement>>();
		for (String key : effectTracks.keySet()) {
			Vector<BarElement> vBarElements = effectTracks.get(key);
			for (int i = 0; i < vBarElements.size(); i++) {
				EffectElement ee = vBarElements.elementAt(i).getEffectElement();
				
				// need to round start and end time before calculating as there are
				// issues with the values and rounding
				
				double startTime = ee.getStartTime();
				double endTime = ee.getEndTime();
				
				System.out.println("B: " + startTime + " -- " + endTime);
				
				BigDecimal bdStartTime = HelperFunctions.round(startTime, 1);
				BigDecimal bdEndTime = HelperFunctions.round(endTime, 1);
				
				System.out.println("A: " + bdStartTime + " -- " + bdEndTime);
				
				BigDecimal keyStartTime = bdStartTime.multiply(new BigDecimal(PTS));
				BigDecimal keyEndTime = bdEndTime.multiply(new BigDecimal(PTS));
				if (mergedHashtable.containsKey(keyStartTime)) {
					Vector<EffectElement> container = mergedHashtable.get(keyStartTime);
					
					boolean found = false;
					for (int j = 0; j < container.size() && !found; j++) {
						EffectElement ee2 = container.elementAt(j);
						if (ee2.getEffectName() != null && ee2.getEffectName().equals(ee.getEffectName()))
							found = true;
					}
					
					if (!found)
						container.add(ee);
				}
				else {
					Vector<EffectElement> container = new Vector<EffectElement>();
					container.add(ee);
					mergedHashtable.put(keyStartTime, container);
				}
				
				if (mergedHashtable.containsKey(keyEndTime)) {
					Vector<EffectElement> container = mergedHashtable.get(keyEndTime);
					
					boolean found = false;
					for (int j = 0; j < container.size() && !found; j++) {
						EffectElement ee2 = container.elementAt(j);
						if (ee2.getEffectName() != null && ee2.getEffectName().equals(ee.getEffectName()))
							found = true;
					}
					
					if (!found)
						container.add(ee);
				}
				else {
					Vector<EffectElement> container = new Vector<EffectElement>();
					container.add(ee);
					mergedHashtable.put(keyEndTime, container);
				}
			}			
		}
		
		return mergedHashtable;
	}
}
