/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

package lib;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;
import javax.swing.border.Border;

import app.SEVinoView;

/**
 * Original ruler written by Scott Violet (taken from: http://java.sun.com/products/jfc/tsc/articles/text/tabs/)
 * strongly modified because not needing tabs and other stuff
 * 
 * @author Markus Waltl
 */
public class Ruler extends JPanel implements MouseListener {
	private static final long serialVersionUID = -3027767331560767384L;
	
	private static SEVinoView gui_ = null;
	// Some defines used in painting the ruler.
	protected static final int DPI = 70;
	protected static final int H_DPI = DPI / 2;
	protected static final int E_DPI = DPI / 10;

	/** Shared instance of default border. */
	protected static final Border DefaultBorder = new RulerBorder();

	/** Font using for the units. */
	private Font unitsFont;
	/** Total font height. */
	private int fontHeight;
	/** Font ascent. */
	private int fontAscent;
	private int passedTime = 0;

	/**
	 * Constructor
	 */
	public Ruler(SEVinoView gui) {
		gui_ = gui;
		setBackground(Color.white);

		Border border = createBorder();

		if (border != null) {
			setBorder(border);
		}
		
		addMouseListener(this);
	}

	/**
	 * Method for retrieving the passed time
	 * @return int with time
	 */
	public int getPassedTime() {
		return passedTime;
	}

	/**
	 * Method for setting the passed time
	 * @param passedTime int with passed time
	 */
	public void setPassedTime(int passedTime) {
		this.passedTime = passedTime;
	}
	
	//
	// Painting methods
	//

	/**
	 * Messaged to paint the Component, will fill the background and 
	 * message paintUnits.
	 */
	protected void paintComponent(Graphics g) {
		super.paintComponent( g );
		Rectangle clip = this.getVisibleRect();
		Insets insets = getInsets();

		updateFontIfNecessary();
		g.setColor(getBackground());
		g.fillRect(clip.x, clip.y, clip.width, clip.height);
		// fill runoff area
		g.setColor(Color.yellow);
		g.fillRect(0, clip.y, passedTime, clip.height);
		paintUnits(g, clip, insets);
	}

	/**
	 * Paints the unit indicators.
	 */
	protected void paintUnits(Graphics g, Rectangle clip, Insets insets) {
		int fontY = getUnitsFontAscent();
		int midY = getUnitsFontHeight() / 2;
		int dpiOffset = 0;

		if (insets != null) {
			fontY += insets.top;
			midY += insets.top;
		}
		g.setColor(getUnitsColor());
		g.setFont(getUnitsFont());

		FontMetrics fm = g.getFontMetrics();

		int maxX = clip.x + clip.width;
		for (int x = (clip.x / DPI * DPI + dpiOffset); x <= maxX; x += E_DPI) {
			int tempX = x - dpiOffset;
			if (tempX % DPI == 0) {
				String numString = "" + (tempX/DPI);
				g.drawString(numString, x - fm.stringWidth(numString) / 2,
						fontY);
			} else if (tempX % H_DPI == 0) {
				g.drawLine(x, midY - 3, x, midY + 3);
			} else {
				g.drawLine(x, midY - 1, x, midY + 1);
			}
		}
	}

	/**
	 * Returns the color to use for the units and ticks.
	 */
	protected Color getUnitsColor() {
		return Color.black;
	}

	/**
	 * Returns the Font to use for the units. Override this to specify a
	 * different font.
	 */
	protected Font getUnitsFont() {
		return getFont();
	}

	//
	// Component methods
	//

	public Dimension getPreferredSize() {
		updateFontIfNecessary();

		Insets insets = getInsets();

		if (insets != null) {
			return new Dimension(insets.left + insets.right + super.getPreferredSize().width, insets.top
					+ insets.bottom + getUnitsFontHeight());
		}
		return new Dimension(super.getPreferredSize().width, getUnitsFontHeight());
	}

	public Dimension getMinimumSize() {
		return getPreferredSize();
	}

	public Dimension getMaximumSize() {
		return getPreferredSize();
	}

	/**
	 * The ascent of the units font.
	 */
	protected int getUnitsFontAscent() {
		return fontAscent;
	}

	/**
	 * Returns the height of the tray.
	 */
	protected int getUnitsFontHeight() {
		return fontHeight;
	}

	/**
	 * Updates font height information.
	 */
	@SuppressWarnings("deprecation")
	private void updateFontIfNecessary() {
		Font font = getUnitsFont();

		if (unitsFont != font) {
			fontHeight = fontAscent = 0;
			if (font != null) {
				Toolkit tk = getToolkit();

				if (tk != null) {
					FontMetrics fm = tk.getFontMetrics(font);

					if (fm != null) {
						fontHeight = fm.getHeight();
						fontAscent = fm.getAscent();
						unitsFont = font;
					}
				}
			}
		}
	}

	/**
	 * Returns the default border to use.
	 */
	protected Border createBorder() {
		return DefaultBorder;
	}

	public void mouseClicked(MouseEvent arg0) {	}

	public void mouseEntered(MouseEvent arg0) {	}

	public void mouseExited(MouseEvent arg0) {	}

	public void mousePressed(MouseEvent arg0) {	}

	public void mouseReleased(MouseEvent arg0) {
		gui_.videoPanel.setPlayerTime((double)(arg0.getX()/70));
	}
}
