/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

package lib;

import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import app.SEVinoView;

/**
 * Class for showing slider tooltips 
 * Tooltips during user dragging (thanks to Santhosh Kumar's blog entry: ToolTips can say more (July 04, 2005))
 * 
 * @author Waltl Markus
 * @version 1.0
 */
public class SliderToolTips {
	/**
	 * Method for enabling the slider tooltips
	 * @param slider JSlider to enable tooltips for
	 * @param gui SEVinoView with the GUI
	 */
	public static void enableSliderToolTips(final JSlider slider, final SEVinoView gui) {
		if (slider != null && gui != null) {
			slider.addChangeListener(new ChangeListener() {
				private boolean adjusting = false;
				private String oldTooltip;
				public void stateChanged(ChangeEvent e) {
					boolean needGuiUpdater = false;
					if (slider.getModel().getValueIsAdjusting()) {
						if (!adjusting) {
							oldTooltip = slider.getToolTipText();
							adjusting = true;
						}
						
						if (slider.getName().equals("timeSlider")) {
							slider.setToolTipText(String.valueOf(slider.getValue()/10.0));
							needGuiUpdater = true;
						}
						else if (slider.getName().equals("volumeSlider")) {
							slider.setToolTipText(String.valueOf(slider.getValue()) + "%");
							needGuiUpdater = false;
						}
			
						hideToolTip(slider);
						postToolTip(slider);
						
						if (gui != null && gui.guiUpdater_ != null && needGuiUpdater)
							gui.guiUpdater_.setUserChange(true);
					}
					else {
						hideToolTip(slider);
						slider.setToolTipText(oldTooltip);
						adjusting = false;
						oldTooltip = null;
						
						if (gui != null && gui.videoPanel != null) {
							
							if (slider.getName().equals("timeSlider")) {
								gui.videoPanel.setPlayerTime((double)(slider.getValue()/10));
								needGuiUpdater = true;
							}
							else if (slider.getName().equals("volumeSlider")) {
								int volume = (int)slider.getValue();
						        gui.videoPanel.setVolume(volume);
								needGuiUpdater = false;
							}
							
					        if (gui.guiUpdater_ != null && needGuiUpdater)
								gui.guiUpdater_.setUserChange(false);
						}
					}
				}
			});
		}
	}
	
	/**
	 * Method for posting the tooltip
	 * @param comp JComponent to post for
	 */
	public static void postToolTip(JComponent comp) {
		Action action = comp.getActionMap().get("postTip");
		if (action == null)
			return;
		
		ActionEvent ae = new ActionEvent(comp, ActionEvent.ACTION_PERFORMED, "postTip", EventQueue.getMostRecentEventTime(), 0);
		action.actionPerformed(ae);
	}
	
	/**
	 * Method for hiding the tooltip
	 * @param comp JComponent to hide for
	 */
	public static void hideToolTip(JComponent comp) {
		Action action = comp.getActionMap().get("hideTip");
		if (action == null)
			return;
		
		ActionEvent ae = new ActionEvent(comp, ActionEvent.ACTION_PERFORMED, "hideTip", EventQueue.getMostRecentEventTime(), 0);
		action.actionPerformed(ae);
	}
}
