/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

package lib;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import uk.co.caprica.vlcj.binding.LibVlc;
import uk.co.caprica.vlcj.binding.internal.libvlc_media_t;
import uk.co.caprica.vlcj.player.MediaPlayer;
import uk.co.caprica.vlcj.player.MediaPlayerEventListener;
import uk.co.caprica.vlcj.player.MediaPlayerFactory;
import uk.co.caprica.vlcj.player.embedded.EmbeddedMediaPlayer;
import uk.co.caprica.vlcj.player.embedded.videosurface.CanvasVideoSurface;
import uk.co.caprica.vlcj.runtime.RuntimeUtil;
import uk.co.caprica.vlcj.runtime.windows.WindowsRuntimeUtil;

/**
 * VideoPainter
 * 
 * Class for representing a video panel
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public class VideoPainter extends JPanel {
	private static final long serialVersionUID = -4118435394104958406L;
	private EmbeddedMediaPlayer player_ = null;
	private MediaPlayerFactory factory_ = null;
	private CanvasVideoSurface videoSurface = null;
	private Canvas canvas_ = null;
	private int videoWidth = 0;
	private int videoHeight = 0;
	private Object syncPoint = new Object();
	private String mediaFile_ = "";
	private double curPlayTime_ = 0.0;
	private int playerState_ = 0;
	private Timer timer_ = null;
	
	// States of the player
	public final static int STOPPED = 0;
	public final static int PLAYING = 1;
	public final static int PAUSED = 2;
	public final static int FINISHED = 3;
	
	/**
	 * Constructor
	 */
	public VideoPainter() {
		System.out.println("Infos about VLC:");
		System.out.println("  version: " + LibVlc.INSTANCE.libvlc_get_version());
		System.out.println(" compiler: " + LibVlc.INSTANCE.libvlc_get_compiler());
		System.out.println("changeset: " + LibVlc.INSTANCE.libvlc_get_changeset());
	}
	
	/**
	 * Method for stopping the player
	 */
	public void stop() {
		if (player_ != null) {
			player_.stop();
			curPlayTime_ = 0.0;
		}
	}
	
	/**
	 * Method for rewinding the player
	 */
	public void rewind() {
		if (player_ != null) {
			player_.setPosition(0.0f);
			curPlayTime_ = 0.0;
		}
	}
	
	/**
	 * Method for moving the video 5 seconds backward
	 */
	public void backward() {
		if (player_ != null) {
			player_.skip(-5000);
			curPlayTime_ = curPlayTime_ - 5;
		}
	}
	
	/**
	 * Method for moving the video 5 seconds forward
	 */
	public void forward() {
		if (player_ != null) {
			player_.skip(5000);
			curPlayTime_ = curPlayTime_ + 5;
		}
	}
	
	/**
	 * Method for pausing the video
	 */
	public void pause() {
		if (player_ != null) {
			player_.pause();
		}
	}
	
	/**
	 * Method for destroying the player and controller
	 */
	public void destroyVideo() {
		if(player_==null)
			return;

		if (player_.isPlaying())
			player_.stop();
		
		player_.release();
		player_ = null;
		
		if (factory_ == null)
			return;
			
		factory_.release();
		factory_ = null;
	}
	
	/**
	 * Method for initializing the player
	 * @param mediaFile String with the video to play
	 */
	public boolean init(String mediaFile) {
		if (mediaFile == null) {
			JOptionPane.showMessageDialog(null, "There is no A/V file!");
			return false;
		}
		
		if (timer_ != null) {
		  timer_.cancel();
		  timer_ = null;
		}
		
		curPlayTime_ = 0.0;
		
		if (canvas_ != null) {
			if (player_ != null) {
				if (player_.isPlaying())
					player_.stop();
				
				player_.release();
				player_ = null;
			}
			this.remove(canvas_);
			canvas_ = null;
		}
		
		canvas_ = new Canvas();
		canvas_.setBackground(Color.black);
		this.add(canvas_);
		Dimension videoSize = this.getSize();
		videoWidth = videoSize.width-4;
		videoHeight = videoSize.height-4;
		canvas_.setBounds(2, 2, videoWidth, videoHeight);
		
		// Special case to help out users on Windows...
		List<String> vlcArgs = new ArrayList<String>();
		
		if(RuntimeUtil.isWindows()) {
			vlcArgs.add("--plugin-path=" + WindowsRuntimeUtil.getVlcInstallDir() + "\\plugins");
		}
		
		// disable display of video title
		vlcArgs.add("--no-video-title");
		vlcArgs.add("--no-overlay");

		factory_ = new MediaPlayerFactory(vlcArgs.toArray(new String[vlcArgs.size()]));
		player_ = factory_.newEmbeddedMediaPlayer();
		videoSurface = factory_.newVideoSurface(canvas_);	    
		player_.setVideoSurface(videoSurface);
		
		player_.addMediaPlayerEventListener(new MediaPlayerEventListener() {
			public void finished(MediaPlayer mediaPlayer) {
				//System.out.println("Finished");
				playerState_ = FINISHED;
				curPlayTime_ = 0.0;
			}

			public void paused(MediaPlayer mediaPlayer) {
				//System.out.println("Paused");
				playerState_ = PAUSED;
			}

			public void playing(MediaPlayer mediaPlayer) {
				//System.out.println("Playing");
				playerState_ = PLAYING;
			}

			public void stopped(MediaPlayer mediaPlayer) {
				//System.out.println("Stopped");
				playerState_ = STOPPED;
				curPlayTime_ = 0.0;
			}

			public void backward(MediaPlayer arg0) {}

			public void buffering(MediaPlayer arg0) {}

			public void endOfSubItems(MediaPlayer arg0) {}

			public void error(MediaPlayer arg0) {}

			public void forward(MediaPlayer arg0) {}

			public void lengthChanged(MediaPlayer arg0, long arg1) {}

			public void mediaChanged(MediaPlayer arg0) {}

			public void mediaDurationChanged(MediaPlayer arg0, long arg1) {}

			public void mediaFreed(MediaPlayer arg0) {}

			public void mediaMetaChanged(MediaPlayer arg0, int arg1) {}

			public void mediaParsedChanged(MediaPlayer arg0, int arg1) {}

			public void mediaStateChanged(MediaPlayer arg0, int arg1) {}

			public void mediaSubItemAdded(MediaPlayer arg0, libvlc_media_t arg1) {}

			public void newMedia(MediaPlayer arg0) {}

			public void opening(MediaPlayer arg0) {}

			public void pausableChanged(MediaPlayer arg0, int arg1) {}

			public void positionChanged(MediaPlayer arg0, float arg1) {}

			public void seekableChanged(MediaPlayer arg0, int arg1) {}

			public void snapshotTaken(MediaPlayer arg0, String arg1) {}

			public void subItemFinished(MediaPlayer arg0, int arg1) {}

			public void subItemPlayed(MediaPlayer arg0, int arg1) {}

			public void timeChanged(MediaPlayer arg0, long arg1) {}

			public void titleChanged(MediaPlayer arg0, int arg1) {}
		});
		
		mediaFile_ = mediaFile;
		
		// start internal timer thread
		// workaround because VLC does not return good timestamps!

		timer_ = new Timer();
		timer_.schedule(new TimerTask() {
			public void run() {
				if (player_ != null && player_.isPlaying())
					curPlayTime_ = curPlayTime_ + 0.1;
			}
		}, 0, 100);
		
		return true;
	}
	
	/**
	 * Method for starting the player
	 */
	public void start() {
		if (player_ != null) {
			if (playerState_ == PAUSED)
				player_.play();
			else if (playerState_ == STOPPED ||
					playerState_ == FINISHED)
				player_.playMedia(mediaFile_);
		}
	}
	
	/**
	 * Method for returning the current play time in seconds
	 * @return long with the current play time
	 */
	public double getCurrentVideoTimeSeconds() {
		double retTime = -1.0;
		if (player_ != null) {
			try {
				retTime = player_.getTime() / 1000.0;
			} catch (Exception e) {	}
		}
		
		return retTime;
	}
	
	/**
	 * Method for setting the player time
	 * @param seconds double with the time
	 */
	public void setPlayerTime(double seconds) {
		if (player_ != null) {
			synchronized(syncPoint) {
				try {
					player_.setTime((long)(seconds * 1000));
					curPlayTime_ = seconds;
				} catch (Exception e) {}
			}
		}
	}
	
	/**
	 * Method for getting the video duration
	 * @return long with the duration
	 */
	public long getVideoDuration() {
		if (player_ != null)
			return player_.getLength();
		else
			return 0;
	}
	
	/**
	 * Method for returning the current video play time in seconds
	 * Note: this is internally calculated for using the time
	 * from VLC use getCurrentVideoTimeSeconds()
	 * @return double with the current time
	 */
	public double getCurrentVideoTime() {
		return curPlayTime_;
	}
	
	/**
	 * Method for returning the state of the video player
	 * @return int with the state
	 */
	public int getPlayerState() {
		return playerState_;
	}
	
	/**
	 * Method for retrieving a video frame
	 * @return BufferedImage with video frame
	 */
	public BufferedImage getFrame() {
		if (player_ != null) {
			try {
				return player_.getVideoSurfaceContents();
			} catch (Exception e) {
				return null;
			}
		}
		else
			return null;
	}
	
	/**
	 * Method for setting the player volume
	 * @param volume int with the volume 0 to 200 (200 full volume)
	 */
	public void setVolume(int volume) {
		if (player_ != null) {
			player_.setVolume(volume);
		}
	}
	
	/**
	 * Method for getting the player volume
	 * @return int with the volume 0 to 200 (200 full volume)
	 */
	public int getVolume() {
		int retVal = 0;
		if (player_ != null) {
			retVal = player_.getVolume();
		}
		
		return retVal;
	}
}
