/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package xml;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Vector;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;

import jaxb.configuration.AttributeType;
import jaxb.configuration.AttributeType.Enumeration;
import jaxb.configuration.CommonAttributesType;
import jaxb.configuration.ReferenceAttributeType;
import jaxb.configuration.SEVino;
import jaxb.configuration.Types;
import jaxb.configuration.SEVino.Effect;

import lib.ConfigDefinitions;
import lib.ConfigElement;

/**
 * ConfigHandler
 * 
 * Class for handling the SEVino config content in the XML file
 * 
 * @author Markus Waltl (Alpen-Adria-Unversitaet Klagenfurt)
 * @version 1.0
 */
public class ConfigHandler {
	private static Vector<ConfigElement> buffer_ = null;

	/**
	 * Constructor
	 */
	public ConfigHandler() {
		buffer_ = new Vector<ConfigElement>();
	}
	
	/**
	 * Method for parsing a config file
	 * @param fileToParse path to file to be parsed
	 * @return Vector<ConfigElement> with configuration elements if successful else null
	 * @throws IOException if an I/O exception occurs, e.g., file does not exist
	 * @throws JAXBException if a parsing exception occured
	 */
	public Vector<ConfigElement> parse(String fileToParse) throws IOException, JAXBException {
		JAXBContext jaxbContext;
		try {
			File file = new File(fileToParse);
			if (!file.exists() || !file.canRead())
				throw new IOException("IO error: file does not exist or cannot be read!");
			
			jaxbContext = JAXBContext.newInstance(SEVino.class);
			Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
			JAXBElement<SEVino> annoconf = unmarshaller.unmarshal(new StreamSource(file), SEVino.class);
			
			SEVino ra = annoconf.getValue();
			List<CommonAttributesType> listCommonAtt = ra.getCommonAttributes();
			List<Effect> listEffects = ra.getEffects();
			if (listEffects != null) {
				for (int i = 0; i < listEffects.size(); i++) {
					ConfigElement configElement = new ConfigElement();
					
					Effect effect = listEffects.get(i);
					configElement.setEffectName(effect.getName());
					configElement.setDisplayName(effect.getDisplay());
										
					try {
						String value = effect.getEcolor();
						value = value.toLowerCase();
						//RGB: zB: r10g20b30
						int pos = value.indexOf("g");
						if (pos != -1) {
							int r = Integer.valueOf(value.substring(1, pos));
							int pos2 = value.indexOf("b");
							if (pos2 != -1) {
								int g = Integer.valueOf(value.substring(pos+1, pos2));
								int b = Integer.valueOf(value.substring(pos2+1));
								configElement.setEffectColor(new java.awt.Color(r,g,b));
							}
						}
						
						value = effect.getScolor();
						value = value.toLowerCase();
						//RGB: zB: r10g20b30
						pos = value.indexOf("g");
						if (pos != -1) {
							int r = Integer.valueOf(value.substring(1, pos));
							int pos2 = value.indexOf("b");
							if (pos2 != -1) {
								int g = Integer.valueOf(value.substring(pos+1, pos2));
								int b = Integer.valueOf(value.substring(pos2+1));
								configElement.setSelectColor(new java.awt.Color(r,g,b));
							}
						}
					} catch (NumberFormatException nfe) {}
					
					List<Object> effectAttributes = effect.getAttributesAndReferenceAttributes();
					if (effectAttributes != null) {
						for (Object att : effectAttributes) {
							if (att instanceof ReferenceAttributeType) {
								String commonAttName = ((ReferenceAttributeType)att).getUri();
								
								boolean found = false;
								CommonAttributesType cat = null;
								for (int li = 0; li < listCommonAtt.size() && !found; li++) {
									cat = listCommonAtt.get(li);
									if (cat.getName() != null && cat.getName().equals(commonAttName))										
										found = true;
								}
								
								if (cat != null) {
									List<AttributeType> listCatAtt = cat.getAttributes();
									for (AttributeType catAtt : listCatAtt)
										handleAttribute(configElement, catAtt);
								}
							}
							
							if (att instanceof AttributeType)
								handleAttribute(configElement, (AttributeType)att);
						}					
					}
					buffer_.add(configElement);
				}
				return buffer_;
			}

		} catch (JAXBException e) {
			throw new JAXBException("Parsing error: " + e.getMessage());
		} catch (Exception e) {
			throw new JAXBException("Unexpected error: " + e.getMessage());
		}
		
		return null;
	}
	
	/**
	 * Method for handling the attributes of the config elements
	 * @param configElement ConfigElement object containing the configuration
	 * @param at AttributeType for defining the type of the attribute
	 */
	private void handleAttribute(ConfigElement configElement, AttributeType at) {
		configElement.openNewAttribute();
		String[] attrib = new String[2];
		attrib[0] = ConfigDefinitions.NAME;
		attrib[1] = at.getName();
		configElement.addAttribute(attrib);
		attrib = new String[2];
		attrib[0] = ConfigDefinitions.TYPE;
		attrib[1] = at.getType().value();
		configElement.addAttribute(attrib);
		attrib = new String[2];
		attrib[0] = ConfigDefinitions.TOOLTIP;
		attrib[1] = at.getTooltip();
		configElement.addAttribute(attrib);
									
		if (at.getType() == Types.ENUM || at.getType() == Types.COLOR) {
			List<Enumeration> attribEnum = at.getEnumerations();
			if (attribEnum != null) {
				for (Enumeration enumeration : attribEnum) {
					configElement.openNewEnumeration();
					String[] enumer = new String[2];
					enumer[0] = ConfigDefinitions.DISPLAY;
					enumer[1] = enumeration.getDisplay();
					configElement.addEnumeration(enumer);
					enumer = new String[2];
					enumer[0] = ConfigDefinitions.STANDARD;
					enumer[1] = String.valueOf(enumeration.getStandard());
					configElement.addEnumeration(enumer);										
					configElement.closeEnumeration();
				}
			}
		}
		configElement.closeAttribute();
	}
}
