/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package xml;

import io.UploadFile;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.StringReader;
import java.net.MalformedURLException;
import java.util.Map;
import java.util.StringTokenizer;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;

import org.iso.mpeg.mpegv.sedl.*;

/**
 * Provides means to create information contained in a sensory effect metadata description identified by namespace.
 * Note: this class is based on the reference software for MPEG-V and the MXM engine
 * 
 * @author Markus Waltl
 */

/*
 * TODO: namespaceURI parameters are currently not used!
 */
public class SEMCreator {
	private static SEM semDescription_ = null;
	private static ObjectFactory of = null;
	
	/**
	 * Constructor
	 */
	public SEMCreator() {
		of = new ObjectFactory();
		semDescription_ = of.createSEM();
	}
	
	/**
	 * Provides means to add an effect identified by the namespace namespaceURI.
	 * 
	 * @param namespaceURI The namespaceURI parameter is a String containing the URI of the namespace identifying the effect.
	 * @param effect The effect parameter is a String containing the effect.
	 * @throws JAXBException - if an error occured.
	 */	
	public void addEffect(String namespaceURI, String effect) throws JAXBException {
		try {
			JAXBContext jc = JAXBContext.newInstance(EffectBaseType.class);
			
			// create an Unmarshaller
			Unmarshaller u = jc.createUnmarshaller();
						
			JAXBElement<EffectBaseType> el = u.unmarshal(new StreamSource(new StringReader(effect)), EffectBaseType.class);
			
			semDescription_.getDeclarationsAndGroupOfEffectsAndEffects().add(el.getValue());
		} catch (JAXBException je) {
			je.printStackTrace();
			throw new JAXBException(je.getMessage());
		}		
	}

	/**
	 * Provides means to add a group of effects (goe) identified by the namespace namespaceURI.
	 * 
	 * @param namespaceURI The namespaceURI parameter is a String containing the URI of the namespace identifying the group of effects.
	 * @param goe The goe parameter is a String containing the group of effects.
	 * @throws JAXBException - if an error occured.
	 */
	public void addGroupOfEffects(String namespaceURI, String goe) throws JAXBException {
		try {
			JAXBContext jc = JAXBContext.newInstance(GroupOfEffects.class);
			
			// create an Unmarshaller
			Unmarshaller u = jc.createUnmarshaller();
						
			JAXBElement<GroupOfEffects> el = u.unmarshal(new StreamSource(new StringReader(goe)), GroupOfEffects.class);
			
			semDescription_.getDeclarationsAndGroupOfEffectsAndEffects().add(el.getValue());
		} catch (JAXBException je) {
			je.printStackTrace();
			throw new JAXBException(je.getMessage());
		}		
	}
	
	/**
	 * Provides means to create the description metadata (dm) identified by the namespace namespaceURI.
	 * 
	 * @param namespaceURI The namespaceURI parameter is a String containing the URI of the namespace identifying the description metadata.
	 * @param dm The dm parameter is a String containing the description metadata.
	 * @throws JAXBException - if an error occured.
	 */
	public void setDescriptionMetadata(String namespaceURI, String dm) throws JAXBException {
		try {
			JAXBContext jc = JAXBContext.newInstance(DescriptionMetadataType.class);
			
			// create an Unmarshaller
			Unmarshaller u = jc.createUnmarshaller();
						
			JAXBElement<DescriptionMetadataType> el = u.unmarshal(
					new StreamSource(new StringReader(dm)), DescriptionMetadataType.class);
			
			semDescription_.setDescriptionMetadata(el.getValue());
		} catch (JAXBException je) {
			throw new JAXBException(je.getMessage());
		}		
	}

	/**
	 * Provides means to set the attributes (sa) of a sensory effect metadata description identified by the namespace namespaceURI.
	 * 
	 * @param namespaceURI The namespaceURI parameter is a String containing the URI of the namespace identifying the attributes.
	 * @param sa The sa parameter is a String containing the sem attributes.
	 * @throws JAXBException - if an error occured.
	 */
	public void setSEMAttributes(String namespaceURI, String sa) throws JAXBException {
		StringTokenizer st = new StringTokenizer(sa);
		Map<QName, String> mapping = semDescription_.getOtherAttributes();
		String curString = "";
		String local = "";
		while (st.hasMoreTokens()) {
			String token = st.nextToken();
			String[] s = token.split("=");
			
			if (s.length == 2) {
				if (curString.equals("")) {
					local = s[0];
					curString = s[1];
				}
				else {
					curString = curString.replaceAll("'", "");
					curString = curString.replaceAll("\"", "");
					mapping.put(new QName(local), curString);
					local = s[0];
					curString = s[1];
				}
			}
			else
				curString += " " + s[0];
		}
		
		if (!curString.equals("")) {
			curString = curString.replaceAll("'", "");
			curString = curString.replaceAll("\"", "");
			mapping.put(new QName(local), curString);
		}
	}

	/**
	 * This method is used to create a SEM after effects and settings are set.
	 * @param semPathName a String to where the SEM will be stored.
	 * @param schemaLocation String with the schema locations
	 * @param stylesheetPath String with the path to the XSL
	 * @param validationPath String with the path to the validation XSL
	 * @throws JAXBException - if an error occured.
	 */
	public void createSEM(String semPathName, String schemaLocation, String stylesheetPath //, String validationPath
			) throws JAXBException {
		try {
			FileOutputStream fos = new FileOutputStream(new File(semPathName));
			
			ByteArrayOutputStream baos = (ByteArrayOutputStream)marshal(schemaLocation, stylesheetPath);
			
			fos.write(baos.toByteArray());
			
			fos.flush();
			fos.close();
		} catch (IOException e) {
			throw new JAXBException("IO error: " + e.getMessage());
		} catch (TransformerException e) {
			throw new JAXBException("Transformation error: " + e.getMessage());
		} catch (JAXBException e) {
			throw new JAXBException("Creation error: " + e.getMessage());
		} catch (Exception e) {
			throw new JAXBException("Unexpected error: " + e.getMessage());
		}
	}
	
	/**
	 * This method is used to create and upload a SEM after effects and settings are set.
	 * @param server a String to the server
	 * @param port on the server
	 * @param path path on the server
	 * @param filename to use for transfer
	 * @param schemaLocation String with the schema locations
	 * @param stylesheetPath String with the path to the XSL
	 * @param validationPath String with the path to the validation XSL
	 * @throws JAXBException - if an error occured.
	 */
	public void uploadSEM(String server, int port, String path, String filename, String schemaLocation, String stylesheetPath
			) throws JAXBException {
		try {
			if (server != null && filename != null) {
				ByteArrayOutputStream baos = (ByteArrayOutputStream)marshal(schemaLocation, stylesheetPath);
				
				UploadFile.upload(server, port, path, filename, new ByteArrayInputStream(baos.toByteArray()));
		    }
			else
				System.out.println("Output file or server address not given!");
		} catch (MalformedURLException e) {
			throw new JAXBException("Upload error: " + e.getMessage());
		} catch (IOException e) {
			throw new JAXBException("IO error: " + e.getMessage());
		} catch (TransformerException e) {
			throw new JAXBException("Transformation error: " + e.getMessage());
		} catch (JAXBException e) {
			throw new JAXBException("Creation error: " + e.getMessage());
		} catch (Exception e) {
			throw new JAXBException("Unexpected error: " + e.getMessage());
		}
	}
	
	/**
	 * Method for mashalling SEM content
	 * @param schemaLocation String for the schema location
	 * @param stylesheetPath path to the stylesheet for transformation
	 * @return OutputStream with marshalled SEM
	 * @throws Exception if an error occured
	 */
	private OutputStream marshal(String schemaLocation, String stylesheetPath) throws Exception {
		JAXBContext jaxbContext;
			
		Source styleSheetSource = new StreamSource(new java.io.File(stylesheetPath));
				
		SAXTransformerFactory sf = (SAXTransformerFactory)SAXTransformerFactory.newInstance();
		Transformer transformer = sf.newTransformer(styleSheetSource);

		jaxbContext = JAXBContext.newInstance(SEM.class);
		Marshaller marshaller=jaxbContext.createMarshaller();
		marshaller.setProperty(Marshaller.JAXB_SCHEMA_LOCATION, schemaLocation);
		
		ByteArrayOutputStream docOutputStream = new ByteArrayOutputStream();
		marshaller.marshal(semDescription_, docOutputStream);
		
		// transform output
		ByteArrayInputStream docInputStream = new ByteArrayInputStream(docOutputStream.toByteArray());
		Source src = new StreamSource(docInputStream);
		docOutputStream = new ByteArrayOutputStream();
		Result outputResult = new StreamResult(docOutputStream);
		transformer.transform(src, outputResult);
		
		return docOutputStream;
	}
}
